#######################################################
#
# Family Tree generation program, v2.0
# Written by Ferenc Bodon and Simon Ward, March 2000 (simonward.com)
# Copyright (C) 2000 Ferenc Bodon, Simon K Ward
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# For a copy of the GNU General Public License, visit 
# http://www.gnu.org or write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#######################################################

package DBIFormat;

use strict;
use warnings;
use Person;
use Picture;
use FamilyTreeData;
use Params::Validate qw(:all);
use DataParsers::FieldValidatorParser;
use DataParsers::ExtendedSimonWardFormat; # for getting pictures. Temporal solution
use DBI;

my $picture_directory;

sub set_parent {
  my ($field, $family_tree_data, $temp_person, $parent_type, $parent_type_code) = @_;
  
  if ( defined $field && validIDEntry($field) ) {
        my $parent_id = getID( $field );
        if ( !defined $family_tree_data->{people}{$parent_id} ) {
          my $parent = Person->new({id => $parent_id}); 
          $family_tree_data->{people}{$parent_id} = $parent;            
          $parent->set_name(Name->new(getNameFields($field)));
        } 
        my $a_parent = $family_tree_data->{people}{$parent_id};
        push @{ $family_tree_data->{children}{$parent_id}}, $temp_person;
        
        if( $parent_type eq "father") {
          $temp_person->set_father($a_parent);
        } else {
          $temp_person->set_mother($a_parent);
        }
        
        if( defined $a_parent->get_gender() && 
            $a_parent->get_gender() != $parent_type_code ){
          print "Incorrent gender for ", $a_parent->full_name(), "!?!\n";
        }
        else {
          $a_parent->set_gender($parent_type_code);
        }
   }
}

# return: 0, in case of file open error
sub createFamilyTreeDataFromFile {
  my ($config_) = @_;
  
  my $family_tree_data = FamilyTreeData->new();

  #this may be a security hole!!! config parameters can be obtained directly!!!
  my $datasource_name = $config_->{datasource_name} 
    or die "No datasource_name is given in config";
  my $db_user_name = $config_->{db_user_name}
    or die "No db_user_name is given in config";
  my $db_password = $config_->{db_password}
    or die "No db_password is given in config";
  my $db_table = $config_->{db_table}
    or die "No db_table is given in config";
    
  my $dbh = DBI->connect($datasource_name, $db_user_name, $db_password)
    or die "Couldn't connect to database: " . DBI->errstr;    
  my $sth = $dbh->prepare('SELECT * FROM ' . $db_table)
    or die "Couldn't prepare statement: " . $dbh->errstr;
  $sth->execute() or die "Couldn't execute statement: " . $sth->errstr;    
  while (my $row_href = $sth->fetchrow_hashref){
    my $id = $row_href->{$config_->{mappings}{id}};
    if ( validIDEntry($id) ) {
      $family_tree_data->{people}{ $id } = Person->new( {id => $id} )
        if ( !defined $family_tree_data->{people}{ $id } );
      my $temp_person = $family_tree_data->{people}{ $id };
      
      my ($first_name, $mid_name, $last_name);
      $first_name = $row_href->{$config_->{mappings}{first_name}};
      $mid_name = $row_href->{$config_->{mappings}{mid_name}};
      $last_name = $row_href->{$config_->{mappings}{mid_name}};
      
      $temp_person->{name}->set_name(Name->new(
          {first_name => $first_name,
           mid_name   => $mid_name,
           last_name  => $last_name}));
      my $title = $row_href->{$config_->{mappings}{title}};
      $temp_person->get_name()->set_title($title) if(defined $title);
      
      my $father_id = $row_href->{$config_->{mappings}{father_id}};
      set_parent($father_id, $family_tree_data, $temp_person, "father", 0);
      my $mother_id = $row_href->{$config_->{mappings}{mother_id}};
      set_parent($mother_id, $family_tree_data, $temp_person, "mother", 1);
      
      my $email = $row_href->{$config_->{mappings}{email}};
      $temp_person->set_email($email)  
      	if(FieldValidatorParser::validEmail($email));
    } else {
      print "$id is not a valid ID. Row:" . join(' ', values(%$row_href)) . "  ignored!\n";
    }    
  }
  $sth->finish;
  $dbh->disconnect;
        
  my $picture_directory =  $config_->{photo_dir} 
    or die "No photo_dir is given in config";
  ExtendedSimonWardFormat::setPictureDirectory($picture_directory);
  ExtendedSimonWardFormat::fill_up_pictures($family_tree_data);

  return $family_tree_data;
}

1;

