#######################################################
#
# Family Tree generation program, v2.0
# Written by Ferenc Bodon and Simon Ward, March 2000 (simonward.com)
# Copyright (C) 2000 Ferenc Bodon, Simon K Ward
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# For a copy of the GNU General Public License, visit 
# http://www.gnu.org or write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#######################################################

package GedcomFormat;
use strict;
use warnings;
use Gedcom;
use DataParsers::FieldValidatorParser;
use FamilyTreeData;
use Person;
use Place;
use Cemetery;

my %genderTogedcomsex = (
  U => undef,
  F => 1,
  M => 0,
);
 
sub createFamilyTreeDataFromFile {
  my ($config_) = @_;
  my $file_name = $config_->{file_name} or die "No file_name is given in config";

  my $ged = Gedcom->new(gedcom_file  => $file_name);
  my $family_tree_data = FamilyTreeData->new();                        
  for my $i ($ged->individuals)
  {
    $family_tree_data->{people}{ $i->{xref} } = Person->#!/usr/bin/perl -wnew( $i->{xref} )
        	if ( !defined $family_tree_data->{people}{ $i->{xref} } );
    my $temp_person = $family_tree_data->{people}{ $i->{xref} };
    $temp_person->set_name( Name->new({first_name => $i->given_names(0), 
                                       mid_name   => undef, 
                                       last_name  => $i->surname}));
    if (defined $i->father) {
      $family_tree_data->{people}{ $i->father->{xref} } = Person->new( $i->father->{xref} )
        	if ( !defined $family_tree_data->{people}{ $i->father->{xref} } );
      $temp_person->set_father($family_tree_data->{people}{ $i->father->{xref} });
      push @{ $family_tree_data->{children}{$i->father->{xref}}}, $temp_person;
    }
    if (defined $i->mother) {
      $family_tree_data->{people}{ $i->mother->{xref} } = Person->new( $i->mother->{xref} )
        	if ( !defined $family_tree_data->{people}{ $i->mother->{xref} } );
      $temp_person->set_mother($family_tree_data->{people}{ $i->mother->{xref} });
      push @{ $family_tree_data->{children}{$i->mother->{xref}}}, $temp_person;
    }
    $temp_person->set_gender($genderTogedcomsex{$i->sex()});
    $temp_person->set_date_of_birth( getDate("birth date", $i) );
    $temp_person->set_date_of_death( getDate("death date", $i) );
    $temp_person->set_place_of_birth( getPlace("birth place", $i) );
    $temp_person->set_place_of_death( getPlace("death place", $i) );
    $temp_person->set_cemetery( getCemetery($i) );
    $temp_person->set_email( getEmail($i) );
    $temp_person->set_homepage( getHomepage($i) );
   }
  return $family_tree_data;                         
}

sub getDate {
  my ($date_type, $i) = @_;
    my $date = $i->get_value($date_type);
    if(defined $date) {
#      print $date, "\n";
      $date = (split(/, /, $date))[-1];
      $date =~ tr/[ ]/[\/]/;
      return FieldValidatorParser::getDate($date);       
    }
    return undef;
}
sub getPlace {
  my ($place_type, $i) = @_;
  my $place =  $i->get_value($place_type);
  if(defined $place) {
      my @place_array = split(/,/, $place);
      return @place_array > 1 ? Place->new($place_array[-1], $place_array[-2]) : 
        Place->new($place_array[-1], undef); 
    }
    else {
      return undef;
    }
}
sub getCemetery {
  my ($i) = @_;
  my $cemetery = $i->get_value("burial place");
  if(defined $cemetery) {
      my @cemetery_array = split(/,/, $cemetery);
      if(@cemetery_array > 2) {
        return Cemetery->new($cemetery_array[-1], $cemetery_array[-2], $cemetery_array[-3]);
      }
      elsif(@cemetery_array > 1) {
        return Cemetery->new($cemetery_array[-1], $cemetery_array[-2], undef);
      }
      else {
        return Cemetery->new($cemetery_array[-1], undef, undef);
      }
    }
    else {
      return undef;
    }
}
sub getEmail {
  my ($i) = @_;
  my @data = $i->get_value("Object File");
  my @emails = grep {defined $_ && FieldValidatorParser::validEmail($_)} @data; 
  return $emails[0];

}
sub getHomepage {
  my ($i) = @_;
  my @data = $i->get_value("Object File");
  my @urls = grep {defined $_ && FieldValidatorParser::validURL($_)} @data; 
  return $urls[0];

}

1;