#!/usr/bin/perl -w

use strict;
use warnings;

package FamilyTreeData;

my @parent_array = ("father", "mother");

sub new {
  my ( $classname) = @_;
  my $self = {
    people   => {},  #hash of Person
    children => {}   #hash of childres's array
  };
  return bless $self, $classname;
}
#######################################################
# returns the no. children of this person
sub numChildren {
  my ($self, $person) = @_;
  if (defined $self->{children}{ $person->get_id() }) {
    return ( scalar @{ $self->{children}{ $person->get_id() } } );
  }
  else {
    return 0;
  }
}

sub node_defined {
  my ( $self, $id ) = @_;
  if( defined $self->{children}{ $id } ) {
    return 1;
  }
  else {
    return 0;
  }
}

sub get_father_full_name {
  my ( $self, $id ) = @_;
  if( defined $self->{people}{ $id }->get_father() ) {
  	return $self->{people}{ $id }->get_father()->get_name()->full_name();
  }
  else {
    return "UNKNOWN";
  }
}

sub get_mother_full_name {
  my ( $self, $id ) = @_;
  if( defined $self->{people}{ $id }->get_mother() ) {
  	return $self->{people}{ $id }->get_mother()->get_name()->full_name();
  }
  else {
    return "UNKNOWN";
  }
}
sub get_spouses {
  my ($self, $person) = @_;
  my $spouse_set = Set::Scalar->new;
  my $parent_1;
  my $parent_2;
  if($person->get_gender() == 0){       # male
    $parent_1 = \&Person::get_father;
    $parent_2 = \&Person::get_mother;
  } else {
    $parent_1 = \&Person::get_mother;
    $parent_2 = \&Person::get_father;
  }
  
  foreach my $child (@{$self->{children}{$person->get_id()}}) {
    $spouse_set->insert($parent_2->($child))
      if($parent_1->($child) == $person && defined $parent_2->($child));
  }
  return $spouse_set;
}
sub get_peers {
  my ( $self, $person ) = @_;
  if (defined $person->get_mother()) {
    return grep { (!defined $_->get_father() && !defined $person->get_father()) ||
                  ($_->get_father() == $person->get_father())} 
      @{$self->{children}{$person->get_mother()->get_id()}};
  }
  elsif (defined $person->get_father()) {
    return grep { !defined $_->get_mother() } 
      @{$self->{children}{$person->get_father()->get_id()}};
  }
  else {
    return ($person);
  }  
}
sub get_soft_peers {
  my ( $self, $person, $parent_type ) = @_;
  
  my $parent_func;
  my $other_parent_func;
  if( $parent_type eq "mother" ) {
    $parent_func = \&Person::get_mother;
    $other_parent_func = \&Person::get_father;
  } else {
    $parent_func = \&Person::get_father;
    $other_parent_func = \&Person::get_mother;
  }
  
  if ( defined $parent_func->($person) ) {
    grep {(!defined $other_parent_func->($_) && defined $other_parent_func->($person)) ||
          (defined $other_parent_func->($_) && !defined $other_parent_func->($person)) ||
          (defined $other_parent_func->($_) && defined $other_parent_func->($person) && 
           $other_parent_func->($_) != $other_parent_func->($person)) } 
      @{$self->{children}{$parent_func->($person)->get_id()}};
  }
  else{
    return ();
  }
}



1;